# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.not_found_error import NotFoundError
from ..commons.errors.server_error import ServerError
from ..commons.types.error_message import ErrorMessage
from ..conversation.types.conversation_filter import ConversationFilter
from ..conversation.types.feedback_filter import FeedbackFilter
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..users.types.agent_user_filter import AgentUserFilter
from .types.agent_user_column_definition import AgentUserColumnDefinition
from .types.agent_user_table_response import AgentUserTableResponse
from .types.chart_response import ChartResponse
from .types.conversation_chart_request import ConversationChartRequest
from .types.conversation_column_definition import ConversationColumnDefinition
from .types.conversation_group_by import ConversationGroupBy
from .types.conversation_table_response import ConversationTableResponse
from .types.feedback_column_definition import FeedbackColumnDefinition
from .types.feedback_group_by import FeedbackGroupBy
from .types.feedback_table_response import FeedbackTableResponse
from .types.time_interval import TimeInterval

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAnalyticsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_conversation_table(
        self,
        *,
        field_groupings: typing.Sequence[ConversationGroupBy],
        column_definitions: typing.Sequence[ConversationColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        conversation_filter: typing.Optional[ConversationFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ConversationTableResponse]:
        """
        Retrieves structured conversation data formatted as a table, allowing users to group, filter, and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[ConversationGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row. |
            Note: The field `CreatedAt` should not be used here, all time-based grouping should be done using the `timeGrouping` field.

        column_definitions : typing.Sequence[ConversationColumnDefinition]
            Specifies the metrics to be displayed as columns. Column headers act as keys, with computed metric values as their mapped values. There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly  based on the time they were created. Example: If set to "DAY," data will be aggregated by day.

        conversation_filter : typing.Optional[ConversationFilter]
            Optional filter applied to refine the conversation data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationTableResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/tables/conversations",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "fieldGroupings": convert_and_respect_annotation_metadata(
                    object_=field_groupings, annotation=typing.Sequence[ConversationGroupBy], direction="write"
                ),
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions,
                    annotation=typing.Sequence[ConversationColumnDefinition],
                    direction="write",
                ),
                "conversationFilter": convert_and_respect_annotation_metadata(
                    object_=conversation_filter, annotation=ConversationFilter, direction="write"
                ),
                "timezone": timezone,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationTableResponse,
                    parse_obj_as(
                        type_=ConversationTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_conversation_chart(
        self, *, request: ConversationChartRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ChartResponse]:
        """
        Fetches conversation data visualized in a chart format. Supported chart types include pie chart, date histogram, and stacked bar charts.

        Parameters
        ----------
        request : ConversationChartRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChartResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/charts/conversations",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ConversationChartRequest, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChartResponse,
                    parse_obj_as(
                        type_=ChartResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_feedback_table(
        self,
        *,
        field_groupings: typing.Sequence[FeedbackGroupBy],
        column_definitions: typing.Sequence[FeedbackColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        feedback_filter: typing.Optional[FeedbackFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[FeedbackTableResponse]:
        """
        Retrieves structured feedback data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[FeedbackGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row.
            Note: The field CreatedAt should not be used here, all the time-based grouping should be done using the timeGrouping field.

        column_definitions : typing.Sequence[FeedbackColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Column headers act as keys, with computed metric values as their mapped values.
            There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        feedback_filter : typing.Optional[FeedbackFilter]
            Optional filter applied to refine the feedback data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FeedbackTableResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/tables/feedback",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "fieldGroupings": convert_and_respect_annotation_metadata(
                    object_=field_groupings, annotation=typing.Sequence[FeedbackGroupBy], direction="write"
                ),
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions, annotation=typing.Sequence[FeedbackColumnDefinition], direction="write"
                ),
                "feedbackFilter": convert_and_respect_annotation_metadata(
                    object_=feedback_filter, annotation=FeedbackFilter, direction="write"
                ),
                "timezone": timezone,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FeedbackTableResponse,
                    parse_obj_as(
                        type_=FeedbackTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_agent_user_table(
        self,
        *,
        column_definitions: typing.Sequence[AgentUserColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        agent_user_filter: typing.Optional[AgentUserFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentUserTableResponse]:
        """
        Retrieves structured agent user data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        column_definitions : typing.Sequence[AgentUserColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Only the `count` metric is supported for agent user tables, so each table will have a single column definition using `count`.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        agent_user_filter : typing.Optional[AgentUserFilter]
            Optional filter applied to refine the agent user data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentUserTableResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/tables/agent-users",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions, annotation=typing.Sequence[AgentUserColumnDefinition], direction="write"
                ),
                "agentUserFilter": convert_and_respect_annotation_metadata(
                    object_=agent_user_filter, annotation=AgentUserFilter, direction="write"
                ),
                "timezone": timezone,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentUserTableResponse,
                    parse_obj_as(
                        type_=AgentUserTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAnalyticsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_conversation_table(
        self,
        *,
        field_groupings: typing.Sequence[ConversationGroupBy],
        column_definitions: typing.Sequence[ConversationColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        conversation_filter: typing.Optional[ConversationFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ConversationTableResponse]:
        """
        Retrieves structured conversation data formatted as a table, allowing users to group, filter, and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[ConversationGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row. |
            Note: The field `CreatedAt` should not be used here, all time-based grouping should be done using the `timeGrouping` field.

        column_definitions : typing.Sequence[ConversationColumnDefinition]
            Specifies the metrics to be displayed as columns. Column headers act as keys, with computed metric values as their mapped values. There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly  based on the time they were created. Example: If set to "DAY," data will be aggregated by day.

        conversation_filter : typing.Optional[ConversationFilter]
            Optional filter applied to refine the conversation data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationTableResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/tables/conversations",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "fieldGroupings": convert_and_respect_annotation_metadata(
                    object_=field_groupings, annotation=typing.Sequence[ConversationGroupBy], direction="write"
                ),
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions,
                    annotation=typing.Sequence[ConversationColumnDefinition],
                    direction="write",
                ),
                "conversationFilter": convert_and_respect_annotation_metadata(
                    object_=conversation_filter, annotation=ConversationFilter, direction="write"
                ),
                "timezone": timezone,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationTableResponse,
                    parse_obj_as(
                        type_=ConversationTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_conversation_chart(
        self, *, request: ConversationChartRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ChartResponse]:
        """
        Fetches conversation data visualized in a chart format. Supported chart types include pie chart, date histogram, and stacked bar charts.

        Parameters
        ----------
        request : ConversationChartRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChartResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/charts/conversations",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ConversationChartRequest, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChartResponse,
                    parse_obj_as(
                        type_=ChartResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_feedback_table(
        self,
        *,
        field_groupings: typing.Sequence[FeedbackGroupBy],
        column_definitions: typing.Sequence[FeedbackColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        feedback_filter: typing.Optional[FeedbackFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[FeedbackTableResponse]:
        """
        Retrieves structured feedback data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[FeedbackGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row.
            Note: The field CreatedAt should not be used here, all the time-based grouping should be done using the timeGrouping field.

        column_definitions : typing.Sequence[FeedbackColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Column headers act as keys, with computed metric values as their mapped values.
            There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        feedback_filter : typing.Optional[FeedbackFilter]
            Optional filter applied to refine the feedback data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FeedbackTableResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/tables/feedback",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "fieldGroupings": convert_and_respect_annotation_metadata(
                    object_=field_groupings, annotation=typing.Sequence[FeedbackGroupBy], direction="write"
                ),
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions, annotation=typing.Sequence[FeedbackColumnDefinition], direction="write"
                ),
                "feedbackFilter": convert_and_respect_annotation_metadata(
                    object_=feedback_filter, annotation=FeedbackFilter, direction="write"
                ),
                "timezone": timezone,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FeedbackTableResponse,
                    parse_obj_as(
                        type_=FeedbackTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_agent_user_table(
        self,
        *,
        column_definitions: typing.Sequence[AgentUserColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        agent_user_filter: typing.Optional[AgentUserFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentUserTableResponse]:
        """
        Retrieves structured agent user data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        column_definitions : typing.Sequence[AgentUserColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Only the `count` metric is supported for agent user tables, so each table will have a single column definition using `count`.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        agent_user_filter : typing.Optional[AgentUserFilter]
            Optional filter applied to refine the agent user data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentUserTableResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/tables/agent-users",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions, annotation=typing.Sequence[AgentUserColumnDefinition], direction="write"
                ),
                "agentUserFilter": convert_and_respect_annotation_metadata(
                    object_=agent_user_filter, annotation=AgentUserFilter, direction="write"
                ),
                "timezone": timezone,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentUserTableResponse,
                    parse_obj_as(
                        type_=AgentUserTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .document_information import DocumentInformation
from .inbox_item_base import InboxItemBase
from .inbox_item_fix_deactivate_document import InboxItemFixDeactivateDocument


class InboxItemDuplicateDocuments(InboxItemBase):
    recommended_fixes: typing_extensions.Annotated[
        typing.List[InboxItemFixDeactivateDocument], FieldMetadata(alias="recommendedFixes")
    ] = pydantic.Field()
    """
    The fix recommended for being applied
    """

    other_fixes: typing_extensions.Annotated[
        typing.List[InboxItemFixDeactivateDocument], FieldMetadata(alias="otherFixes")
    ] = pydantic.Field()
    """
    List of fixes associated with the inbox item.
    """

    source_document: typing_extensions.Annotated[DocumentInformation, FieldMetadata(alias="sourceDocument")] = (
        pydantic.Field()
    )
    """
    Information about the source document associated with the inbox item.
    """

    documents: typing.List[DocumentInformation] = pydantic.Field()
    """
    List of Document information objects related to the inbox item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .commit_asset_upload_request import CommitAssetUploadRequest
from .initiate_asset_upload_request import InitiateAssetUploadRequest
from .initiate_asset_upload_response import InitiateAssetUploadResponse

__all__ = ["CommitAssetUploadRequest", "InitiateAssetUploadRequest", "InitiateAssetUploadResponse"]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..commons.types.inbox_item import InboxItem
from ..commons.types.inbox_item_fix import InboxItemFix
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawInboxClient, RawInboxClient
from .types.add_document_fix_request import AddDocumentFixRequest
from .types.inbox_filter import InboxFilter
from .types.inbox_search_response import InboxSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class InboxClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawInboxClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawInboxClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawInboxClient
        """
        return self._raw_client

    def search(
        self,
        *,
        filter: typing.Optional[InboxFilter] = OMIT,
        sort_id: typing.Optional[str] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InboxSearchResponse:
        """
        Retrieve a paginated list of inbox items for an agent.

        Parameters
        ----------
        filter : typing.Optional[InboxFilter]

        sort_id : typing.Optional[str]
            The field to sort by, defaults to created timestamp

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InboxSearchResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.inbox.search()
        """
        _response = self._raw_client.search(
            filter=filter, sort_id=sort_id, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    def get(
        self, inbox_item_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> InboxItem:
        """
        Retrieve details of a specific inbox item by its ID.

        Parameters
        ----------
        inbox_item_id : str
            The ID of the inbox item to get. All other entity ID fields are inferred from the request.

        app_id : str
            The App ID of the inbox item to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InboxItem
            Details of the specified inbox item.

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.inbox.get(
            inbox_item_id="inboxItemId",
            app_id="appId",
        )
        """
        _response = self._raw_client.get(inbox_item_id, app_id=app_id, request_options=request_options)
        return _response.data

    def get_fix(
        self, inbox_item_fix_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> InboxItemFix:
        """
        Retrieve a suggested fix. Includes document information if the fix is a Missing Knowledge suggestion.

        Parameters
        ----------
        inbox_item_fix_id : str
            Unique identifier for the inbox fix.

        app_id : str
            The App ID of the inbox item fix to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InboxItemFix
            Details of the specified inbox item fix.

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.inbox.get_fix(
            inbox_item_fix_id="inboxItemFixId",
            app_id="appId",
        )
        """
        _response = self._raw_client.get_fix(inbox_item_fix_id, app_id=app_id, request_options=request_options)
        return _response.data

    def apply_fixes(
        self,
        inbox_item_id: str,
        *,
        app_id: str,
        fix_reference_ids: typing.Sequence[str],
        add_document_request: typing.Optional[AddDocumentFixRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Apply a list of fixes belonging to an inbox item.

        Parameters
        ----------
        inbox_item_id : str
            Unique identifier for the inbox item.

        app_id : str
            The appId of the inbox item and fixes.

        fix_reference_ids : typing.Sequence[str]
            A list of one or more reference IDs of fixes to apply. All must belong to the same inbox item.

        add_document_request : typing.Optional[AddDocumentFixRequest]
            Only applies to add document fixes, includes the document content to save.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.inbox.apply_fixes(
            inbox_item_id="inboxItemId",
            app_id="appId",
            fix_reference_ids=["fixReferenceIds", "fixReferenceIds"],
        )
        """
        _response = self._raw_client.apply_fixes(
            inbox_item_id,
            app_id=app_id,
            fix_reference_ids=fix_reference_ids,
            add_document_request=add_document_request,
            request_options=request_options,
        )
        return _response.data

    def ignore(
        self, inbox_item_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Ignore a specific inbox item by its ID.

        Parameters
        ----------
        inbox_item_id : str
            Unique identifier for the inbox item.

        app_id : str
            The App ID of the inbox item fix to ignore

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.inbox.ignore(
            inbox_item_id="inboxItemId",
            app_id="appId",
        )
        """
        _response = self._raw_client.ignore(inbox_item_id, app_id=app_id, request_options=request_options)
        return _response.data


class AsyncInboxClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawInboxClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawInboxClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawInboxClient
        """
        return self._raw_client

    async def search(
        self,
        *,
        filter: typing.Optional[InboxFilter] = OMIT,
        sort_id: typing.Optional[str] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InboxSearchResponse:
        """
        Retrieve a paginated list of inbox items for an agent.

        Parameters
        ----------
        filter : typing.Optional[InboxFilter]

        sort_id : typing.Optional[str]
            The field to sort by, defaults to created timestamp

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InboxSearchResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.inbox.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            filter=filter, sort_id=sort_id, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    async def get(
        self, inbox_item_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> InboxItem:
        """
        Retrieve details of a specific inbox item by its ID.

        Parameters
        ----------
        inbox_item_id : str
            The ID of the inbox item to get. All other entity ID fields are inferred from the request.

        app_id : str
            The App ID of the inbox item to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InboxItem
            Details of the specified inbox item.

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.inbox.get(
                inbox_item_id="inboxItemId",
                app_id="appId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(inbox_item_id, app_id=app_id, request_options=request_options)
        return _response.data

    async def get_fix(
        self, inbox_item_fix_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> InboxItemFix:
        """
        Retrieve a suggested fix. Includes document information if the fix is a Missing Knowledge suggestion.

        Parameters
        ----------
        inbox_item_fix_id : str
            Unique identifier for the inbox fix.

        app_id : str
            The App ID of the inbox item fix to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InboxItemFix
            Details of the specified inbox item fix.

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.inbox.get_fix(
                inbox_item_fix_id="inboxItemFixId",
                app_id="appId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_fix(inbox_item_fix_id, app_id=app_id, request_options=request_options)
        return _response.data

    async def apply_fixes(
        self,
        inbox_item_id: str,
        *,
        app_id: str,
        fix_reference_ids: typing.Sequence[str],
        add_document_request: typing.Optional[AddDocumentFixRequest] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Apply a list of fixes belonging to an inbox item.

        Parameters
        ----------
        inbox_item_id : str
            Unique identifier for the inbox item.

        app_id : str
            The appId of the inbox item and fixes.

        fix_reference_ids : typing.Sequence[str]
            A list of one or more reference IDs of fixes to apply. All must belong to the same inbox item.

        add_document_request : typing.Optional[AddDocumentFixRequest]
            Only applies to add document fixes, includes the document content to save.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.inbox.apply_fixes(
                inbox_item_id="inboxItemId",
                app_id="appId",
                fix_reference_ids=["fixReferenceIds", "fixReferenceIds"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.apply_fixes(
            inbox_item_id,
            app_id=app_id,
            fix_reference_ids=fix_reference_ids,
            add_document_request=add_document_request,
            request_options=request_options,
        )
        return _response.data

    async def ignore(
        self, inbox_item_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Ignore a specific inbox item by its ID.

        Parameters
        ----------
        inbox_item_id : str
            Unique identifier for the inbox item.

        app_id : str
            The App ID of the inbox item fix to ignore

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.inbox.ignore(
                inbox_item_id="inboxItemId",
                app_id="appId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.ignore(inbox_item_id, app_id=app_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .attachment_status import AttachmentStatus


class ActionExecutionParamValue_Primitive(UniversalBaseModel):
    value: typing.Optional[typing.Any] = None
    value_type: typing_extensions.Annotated[typing.Literal["primitive"], FieldMetadata(alias="valueType")] = "primitive"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class ActionExecutionParamValue_Attachment(UniversalBaseModel):
    value_type: typing_extensions.Annotated[typing.Literal["attachment"], FieldMetadata(alias="valueType")] = (
        "attachment"
    )
    url: typing.Optional[str] = None
    size_bytes: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="sizeBytes")] = None
    status: AttachmentStatus
    type: str
    name: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ActionExecutionParamValue = typing.Union[ActionExecutionParamValue_Primitive, ActionExecutionParamValue_Attachment]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.not_found_error import NotFoundError
from ..commons.errors.server_error import ServerError
from ..commons.types.error_message import ErrorMessage
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from .types.initiate_asset_upload_response import InitiateAssetUploadResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAssetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def initiate_upload(
        self, *, type: str, name: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[InitiateAssetUploadResponse]:
        """
        Initiate an upload.
        Returns a pre-signed URL for direct file upload and an asset ID for subsequent operations.

        Parameters
        ----------
        type : str
            The mime-type of the attachment. Supported types are:
            - image/jpeg
            - image/jpg
            - image/png
            - image/gif
            - image/webp
            - application/pdf
            - text/plain
            - text/csv
            - application/vnd.openxmlformats-officedocument.wordprocessingml.document
            - application/vnd.openxmlformats-officedocument.spreadsheetml.sheet
            - application/vnd.openxmlformats-officedocument.presentationml.presentation
            - application/msword
            - application/vnd.ms-excel
            - application/vnd.ms-powerpoint
            - audio/aac
            - audio/mpeg
            - audio/mp4
            - audio/wav
            - audio/ogg
            - video/mp4
            - video/webm

        name : typing.Optional[str]
            An optional name for the attachment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InitiateAssetUploadResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/assets",
            method="POST",
            json={
                "type": type,
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InitiateAssetUploadResponse,
                    parse_obj_as(
                        type_=InitiateAssetUploadResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def commit_upload(
        self,
        asset_reference_id: str,
        *,
        checksum: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Commit an upload after successful file transfer.
        Updates the asset status and makes it available for use.

        Parameters
        ----------
        asset_reference_id : str
            The reference ID of the asset to commit (provided by the initiate call). All other entity ID fields are inferred from the API request.

        checksum : typing.Optional[str]
            Checksum of the uploaded file (optional verification)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/assets/{jsonable_encoder(asset_reference_id)}/commit",
            method="POST",
            json={
                "checksum": checksum,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAssetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def initiate_upload(
        self, *, type: str, name: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[InitiateAssetUploadResponse]:
        """
        Initiate an upload.
        Returns a pre-signed URL for direct file upload and an asset ID for subsequent operations.

        Parameters
        ----------
        type : str
            The mime-type of the attachment. Supported types are:
            - image/jpeg
            - image/jpg
            - image/png
            - image/gif
            - image/webp
            - application/pdf
            - text/plain
            - text/csv
            - application/vnd.openxmlformats-officedocument.wordprocessingml.document
            - application/vnd.openxmlformats-officedocument.spreadsheetml.sheet
            - application/vnd.openxmlformats-officedocument.presentationml.presentation
            - application/msword
            - application/vnd.ms-excel
            - application/vnd.ms-powerpoint
            - audio/aac
            - audio/mpeg
            - audio/mp4
            - audio/wav
            - audio/ogg
            - video/mp4
            - video/webm

        name : typing.Optional[str]
            An optional name for the attachment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InitiateAssetUploadResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/assets",
            method="POST",
            json={
                "type": type,
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InitiateAssetUploadResponse,
                    parse_obj_as(
                        type_=InitiateAssetUploadResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def commit_upload(
        self,
        asset_reference_id: str,
        *,
        checksum: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Commit an upload after successful file transfer.
        Updates the asset status and makes it available for use.

        Parameters
        ----------
        asset_reference_id : str
            The reference ID of the asset to commit (provided by the initiate call). All other entity ID fields are inferred from the API request.

        checksum : typing.Optional[str]
            Checksum of the uploaded file (optional verification)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/assets/{jsonable_encoder(asset_reference_id)}/commit",
            method="POST",
            json={
                "checksum": checksum,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from .entity_id import EntityId
from .novel_system_event_no_id import NovelSystemEventNoId


class SystemEvent(NovelSystemEventNoId):
    id: EntityId = pydantic.Field()
    """
    The unique ID of the event
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawAgentCapabilitiesClient, RawAgentCapabilitiesClient
from .types.agent_capability import AgentCapability
from .types.agent_capability_field import AgentCapabilityField
from .types.agent_capability_type import AgentCapabilityType
from .types.execute_capability_response import ExecuteCapabilityResponse
from .types.list_agent_capabilities_response import ListAgentCapabilitiesResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AgentCapabilitiesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAgentCapabilitiesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAgentCapabilitiesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAgentCapabilitiesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        capability_type: typing.Optional[AgentCapabilityType] = OMIT,
        pinned: typing.Optional[bool] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        integration_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        description: typing.Optional[str] = OMIT,
        user_interaction_required: typing.Optional[bool] = OMIT,
        sort_id: typing.Optional[AgentCapabilityField] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListAgentCapabilitiesResponse:
        """
        List all capabilities for an agent.

        Parameters
        ----------
        capability_type : typing.Optional[AgentCapabilityType]

        pinned : typing.Optional[bool]

        enabled : typing.Optional[bool]

        integration_ids : typing.Optional[typing.Sequence[str]]

        description : typing.Optional[str]

        user_interaction_required : typing.Optional[bool]

        sort_id : typing.Optional[AgentCapabilityField]
            The field to sort by, defaults to created timestamp

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListAgentCapabilitiesResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agent_capabilities.list()
        """
        _response = self._raw_client.list(
            capability_type=capability_type,
            pinned=pinned,
            enabled=enabled,
            integration_ids=integration_ids,
            description=description,
            user_interaction_required=user_interaction_required,
            sort_id=sort_id,
            page=page,
            size=size,
            sort_desc=sort_desc,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, integration_id: str, capability_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AgentCapability:
        """
        Parameters
        ----------
        integration_id : str

        capability_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentCapability

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agent_capabilities.get(
            integration_id="integrationId",
            capability_id="capabilityId",
        )
        """
        _response = self._raw_client.get(integration_id, capability_id, request_options=request_options)
        return _response.data

    def patch(
        self,
        integration_id: str,
        capability_id: str,
        *,
        enabled: typing.Optional[bool] = OMIT,
        description_override: typing.Optional[str] = OMIT,
        pinned: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentCapability:
        """
        Parameters
        ----------
        integration_id : str

        capability_id : str

        enabled : typing.Optional[bool]
            Whether this capability is enabled or disabled

        description_override : typing.Optional[str]
            Override description for action capabilities. Only applies to actions, ignored for triggers.

        pinned : typing.Optional[bool]
            Whether this action capability is pinned. Only applies to actions, ignored for triggers.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentCapability

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agent_capabilities.patch(
            integration_id="integrationId",
            capability_id="capabilityId",
        )
        """
        _response = self._raw_client.patch(
            integration_id,
            capability_id,
            enabled=enabled,
            description_override=description_override,
            pinned=pinned,
            request_options=request_options,
        )
        return _response.data

    def execute(
        self,
        integration_id: str,
        capability_id: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExecuteCapabilityResponse:
        """
        Execute an action capability.

        Parameters
        ----------
        integration_id : str

        capability_id : str

        parameters : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Parameters to pass to the action execution

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExecuteCapabilityResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.agent_capabilities.execute(
            integration_id="integrationId",
            capability_id="capabilityId",
        )
        """
        _response = self._raw_client.execute(
            integration_id, capability_id, parameters=parameters, request_options=request_options
        )
        return _response.data


class AsyncAgentCapabilitiesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAgentCapabilitiesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAgentCapabilitiesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAgentCapabilitiesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        capability_type: typing.Optional[AgentCapabilityType] = OMIT,
        pinned: typing.Optional[bool] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        integration_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        description: typing.Optional[str] = OMIT,
        user_interaction_required: typing.Optional[bool] = OMIT,
        sort_id: typing.Optional[AgentCapabilityField] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListAgentCapabilitiesResponse:
        """
        List all capabilities for an agent.

        Parameters
        ----------
        capability_type : typing.Optional[AgentCapabilityType]

        pinned : typing.Optional[bool]

        enabled : typing.Optional[bool]

        integration_ids : typing.Optional[typing.Sequence[str]]

        description : typing.Optional[str]

        user_interaction_required : typing.Optional[bool]

        sort_id : typing.Optional[AgentCapabilityField]
            The field to sort by, defaults to created timestamp

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListAgentCapabilitiesResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agent_capabilities.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            capability_type=capability_type,
            pinned=pinned,
            enabled=enabled,
            integration_ids=integration_ids,
            description=description,
            user_interaction_required=user_interaction_required,
            sort_id=sort_id,
            page=page,
            size=size,
            sort_desc=sort_desc,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, integration_id: str, capability_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AgentCapability:
        """
        Parameters
        ----------
        integration_id : str

        capability_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentCapability

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agent_capabilities.get(
                integration_id="integrationId",
                capability_id="capabilityId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(integration_id, capability_id, request_options=request_options)
        return _response.data

    async def patch(
        self,
        integration_id: str,
        capability_id: str,
        *,
        enabled: typing.Optional[bool] = OMIT,
        description_override: typing.Optional[str] = OMIT,
        pinned: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentCapability:
        """
        Parameters
        ----------
        integration_id : str

        capability_id : str

        enabled : typing.Optional[bool]
            Whether this capability is enabled or disabled

        description_override : typing.Optional[str]
            Override description for action capabilities. Only applies to actions, ignored for triggers.

        pinned : typing.Optional[bool]
            Whether this action capability is pinned. Only applies to actions, ignored for triggers.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentCapability

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agent_capabilities.patch(
                integration_id="integrationId",
                capability_id="capabilityId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(
            integration_id,
            capability_id,
            enabled=enabled,
            description_override=description_override,
            pinned=pinned,
            request_options=request_options,
        )
        return _response.data

    async def execute(
        self,
        integration_id: str,
        capability_id: str,
        *,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExecuteCapabilityResponse:
        """
        Execute an action capability.

        Parameters
        ----------
        integration_id : str

        capability_id : str

        parameters : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Parameters to pass to the action execution

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExecuteCapabilityResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.agent_capabilities.execute(
                integration_id="integrationId",
                capability_id="capabilityId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.execute(
            integration_id, capability_id, parameters=parameters, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id_base import EntityIdBase
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .segment_base import SegmentBase


class SegmentRequest(SegmentBase):
    """
    Examples
    --------
    from mavenagi.commons import EntityIdBase, Precondition_Group, Precondition_User
    from mavenagi.segments import SegmentRequest

    SegmentRequest(
        segment_id=EntityIdBase(
            reference_id="admin-users",
        ),
        name="Admin users",
        precondition=Precondition_Group(
            operator="AND",
            preconditions=[
                Precondition_User(
                    key="userKey",
                ),
                Precondition_User(
                    key="userKey2",
                ),
            ],
        ),
    )
    """

    segment_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="segmentId")] = pydantic.Field()
    """
    ID that uniquely identifies this segment
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from ...commons.types.precondition_group import PreconditionGroup  # noqa: E402, F401, I001



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id import EntityId
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .agent_environment import AgentEnvironment
from .pii_category import PiiCategory


class Agent(UniversalBaseModel):
    agent_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="agentId")] = pydantic.Field()
    """
    ID that uniquely identifies this action
    """

    name: str = pydantic.Field()
    """
    The name of the agent.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    When the agent was created.
    """

    environment: AgentEnvironment = pydantic.Field()
    """
    The environment of the agent. Default is `DEMO`.
    """

    default_timezone: typing_extensions.Annotated[str, FieldMetadata(alias="defaultTimezone")] = pydantic.Field()
    """
    The agent's default timezone. This is used when a timezone is not set on a conversation.
    """

    enabled_pii_categories: typing_extensions.Annotated[
        typing.Set[PiiCategory], FieldMetadata(alias="enabledPiiCategories")
    ] = pydantic.Field()
    """
    The PII categories that are enabled for the agent. 
    PII will be automatically redacted from all conversation message text. 
    Attachments and form submissions are not affected. 
    
    Defaults to `AbaRoutingNumber`, `CreditCardNumber`, `IpAddress`, `PhoneNumber`, `SwiftCode`, 
    `UsBankAccountNumber`, `UsDriversLicenseNumber`, `UsIndividualTaxpayerIdentification`,
    `UsUkPassportNumber`, `UsSocialSecurityNumber`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from . import (
    actions,
    agent_capabilities,
    agents,
    analytics,
    app_settings,
    assets,
    commons,
    conversation,
    events,
    inbox,
    knowledge,
    organizations,
    segments,
    translations,
    triggers,
    users,
)
from .actions import ActionField, ActionFilter, ActionRequest, ActionsResponse, ActionsSearchRequest
from .agent_capabilities import (
    ActionCapability,
    AgentCapability,
    AgentCapabilityBase,
    AgentCapabilityField,
    AgentCapabilityListRequest,
    AgentCapabilityType,
    AgentCapability_Action,
    AgentCapability_Trigger,
    ExecuteCapabilityRequest,
    ExecuteCapabilityResponse,
    ListAgentCapabilitiesResponse,
    PatchAgentCapabilityRequest,
    TriggerCapability,
    TriggerType,
)
from .agents import (
    Agent,
    AgentEnvironment,
    AgentField,
    AgentFilter,
    AgentPatchRequest,
    AgentsSearchRequest,
    AgentsSearchResponse,
    CreateAgentRequest,
    PiiCategory,
)
from .analytics import (
    AgentUserAnalyticsRequest,
    AgentUserColumnDefinition,
    AgentUserCount,
    AgentUserMetric,
    AgentUserMetric_Count,
    AgentUserRow,
    AgentUserTableRequest,
    AgentUserTableResponse,
    BarChartResponse,
    CellData,
    CellData_Double,
    CellData_Long,
    CellData_Millisecond,
    CellData_String,
    ChartResponse,
    ChartResponse_BarChart,
    ChartResponse_DateHistogram,
    ChartResponse_PieChart,
    ColumnDefinitionBase,
    ConversationAnalyticsRequest,
    ConversationAverage,
    ConversationBarChartRequest,
    ConversationBasicMetric,
    ConversationChartRequest,
    ConversationChartRequest_BarChart,
    ConversationChartRequest_DateHistogram,
    ConversationChartRequest_PieChart,
    ConversationColumnDefinition,
    ConversationCount,
    ConversationDateHistogramRequest,
    ConversationDistinctCount,
    ConversationGroupBy,
    ConversationMax,
    ConversationMedian,
    ConversationMetric,
    ConversationMetric_Average,
    ConversationMetric_Count,
    ConversationMetric_DistinctCount,
    ConversationMetric_Max,
    ConversationMetric_Median,
    ConversationMetric_Min,
    ConversationMetric_Percentile,
    ConversationMetric_Sum,
    ConversationMin,
    ConversationNumericMetric,
    ConversationPercentile,
    ConversationPieChartRequest,
    ConversationRow,
    ConversationSum,
    ConversationTableRequest,
    ConversationTableResponse,
    DateHistogramResponse,
    FeedbackAnalyticsRequest,
    FeedbackColumnDefinition,
    FeedbackCount,
    FeedbackDistinctCount,
    FeedbackGroupBy,
    FeedbackMetric,
    FeedbackMetric_Count,
    FeedbackMetric_DistinctCount,
    FeedbackRow,
    FeedbackTableRequest,
    FeedbackTableResponse,
    FieldValue,
    FieldValue_Boolean,
    FieldValue_DateTime,
    FieldValue_Double,
    FieldValue_EntityId,
    FieldValue_Long,
    FieldValue_Range,
    FieldValue_String,
    GroupByBase,
    LabeledPoint,
    PieChartResponse,
    Range,
    RowBase,
    Series,
    TableResponseBase,
    TimeDataPoint,
    TimeInterval,
    TimeSeries,
)
from .app_settings import AppSettings, SearchAppSettingsResponse
from .assets import CommitAssetUploadRequest, InitiateAssetUploadRequest, InitiateAssetUploadResponse
from .client import AsyncMavenAGI, MavenAGI
from .commons import (
    ActionBase,
    ActionEnumOption,
    ActionExecutionParamValue,
    ActionExecutionParamValue_Attachment,
    ActionExecutionParamValue_Primitive,
    ActionFormField,
    ActionOAuthConfiguration,
    ActionParameter,
    ActionParameterType,
    ActionResponse,
    AppPrecondition,
    AppUser,
    AppUserIdentifier,
    AppUserIdentifyingPropertyType,
    AppUserRequest,
    AppUserResponse,
    AttachmentRequest,
    AttachmentResponse,
    AttachmentStatus,
    BadRequestError,
    BaseAttachment,
    BaseConversationResponse,
    BasePaginatedRequest,
    BaseSearchRequest,
    BotActionFormResponse,
    BotChartResponse,
    BotConversationMessageType,
    BotLogic,
    BotLogicActionExecutedDetail,
    BotLogicActionItem,
    BotLogicActionReviewedDetail,
    BotLogicFormSubmissionItem,
    BotLogicItem,
    BotLogicItem_Actions,
    BotLogicItem_Form,
    BotLogicItem_Knowledge,
    BotLogicItem_Safety,
    BotLogicItem_Segments,
    BotLogicItem_User,
    BotLogicKnowledgeDetail,
    BotLogicKnowledgeItem,
    BotLogicSafetyItem,
    BotLogicSegmentsItem,
    BotLogicUserItem,
    BotMessage,
    BotOAuthButtonResponse,
    BotObjectResponse,
    BotResponse,
    BotResponseMetadata,
    BotResponse_ActionForm,
    BotResponse_Chart,
    BotResponse_OauthButton,
    BotResponse_Object,
    BotResponse_Text,
    BotTextResponse,
    BrowserInfo,
    BrowserType,
    Capability,
    ChartSpecSchema,
    ContextInfo,
    ConversationAnalysis,
    ConversationExecutedActionPrecondition,
    ConversationInformation,
    ConversationMessageBase,
    ConversationMessageResponse,
    ConversationMessageResponse_Bot,
    ConversationMessageResponse_User,
    ConversationPrecondition,
    ConversationPrecondition_ActionExecuted,
    ConversationPrecondition_App,
    ConversationPrecondition_Metadata,
    ConversationPrecondition_ResponseConfig,
    ConversationPrecondition_Tags,
    ConversationPreview,
    ConversationResponse,
    ConversationSummary,
    DeviceInfo,
    DeviceType,
    DocumentInformation,
    EntityId,
    EntityIdBase,
    EntityIdFilter,
    EntityIdWithoutAgent,
    EntityResult,
    EntityType,
    ErrorMessage,
    EventBaseNoId,
    EventField,
    EventFilter,
    EventRequest,
    EventRequest_SystemEvent,
    EventRequest_UserEvent,
    EventResponse,
    EventResponse_SystemEvent,
    EventResponse_UserEvent,
    EventType,
    EventUserInfo,
    EventUserInfoBase,
    EventsSearchRequest,
    EventsSearchResponse,
    Feedback,
    FeedbackBase,
    FeedbackInfo,
    FeedbackType,
    GeoInfo,
    InboxItem,
    InboxItemBase,
    InboxItemDuplicateDocuments,
    InboxItemFix,
    InboxItemFixAddDocument,
    InboxItemFixBase,
    InboxItemFixDeactivateDocument,
    InboxItemFix_AddDocument,
    InboxItemFix_DeactivateDocument,
    InboxItemMissingKnowledge,
    InboxItemSeverity,
    InboxItemStatus,
    InboxItemType,
    InboxItem_DuplicateDocuments,
    InboxItem_MissingKnowledge,
    IpInfo,
    LanguageInfo,
    LlmInclusionStatus,
    LlmPersona,
    MessageStatus,
    MetadataPrecondition,
    NotFoundError,
    NovelSystemEvent,
    NovelSystemEventNoId,
    NovelUserEvent,
    NumberRange,
    OsInfo,
    OsType,
    Page,
    PageInfo,
    Precondition,
    PreconditionBase,
    PreconditionGroup,
    PreconditionGroupOperator,
    PreconditionOperator,
    Precondition_Conversation,
    Precondition_Group,
    Precondition_User,
    Quality,
    QualityReason,
    ResponseConfig,
    ResponseConfigPrecondition,
    ResponseLength,
    ScopedEntity,
    Sentiment,
    ServerError,
    SessionInfo,
    SimulationContext,
    Source,
    SourceInfo,
    SourceType,
    SurveyInfo,
    SystemEvent,
    SystemEventName,
    TagsPrecondition,
    UserConversationMessageType,
    UserData,
    UserEvent,
    UserEventName,
    UserMessage,
    UserMessageBase,
    UserMessageResponseState,
    VisibilityType,
)
from .conversation import (
    ActionFormAttachment,
    ActionFormRequestParamValue,
    AskObjectRequest,
    AskRequest,
    AskStreamActionEvent,
    AskStreamChartEvent,
    AskStreamEndEvent,
    AskStreamMetadataEvent,
    AskStreamOAuthButtonEvent,
    AskStreamStartEvent,
    AskStreamTextEvent,
    CategorizationResponse,
    ConversationField,
    ConversationFilter,
    ConversationMessageRequest,
    ConversationMetadata,
    ConversationPatchRequest,
    ConversationRequest,
    ConversationsResponse,
    ConversationsSearchRequest,
    DeliverConversationMessageRequest,
    DeliverMessageRequest,
    DeliverMessageRequest_Conversation,
    DeliverMessageRequest_User,
    DeliverMessageResponse,
    DeliverUserMessageRequest,
    DeliveryStatus,
    FeedbackField,
    FeedbackFilter,
    FeedbackRequest,
    NumericConversationField,
    ObjectStreamResponse,
    ObjectStreamResponse_End,
    ObjectStreamResponse_Start,
    ObjectStreamResponse_Text,
    ResolutionStatus,
    SimulationFilter,
    StreamResponse,
    StreamResponse_Action,
    StreamResponse_Chart,
    StreamResponse_End,
    StreamResponse_Metadata,
    StreamResponse_OauthButton,
    StreamResponse_Start,
    StreamResponse_Text,
    SubmitActionFormRequest,
    UpdateMetadataRequest,
)
from .environment import MavenAGIEnvironment
from .inbox import AddDocumentFixRequest, ApplyFixesRequest, InboxFilter, InboxSearchRequest, InboxSearchResponse
from .knowledge import (
    BaseKnowledgeDocument,
    FinalizeKnowledgeBaseVersionRequest,
    KnowledgeBaseField,
    KnowledgeBaseFilter,
    KnowledgeBaseProperties,
    KnowledgeBaseRefreshFrequency,
    KnowledgeBaseRequest,
    KnowledgeBaseResponse,
    KnowledgeBaseSearchRequest,
    KnowledgeBaseType,
    KnowledgeBaseVersion,
    KnowledgeBaseVersionFinalizeStatus,
    KnowledgeBaseVersionRequest,
    KnowledgeBaseVersionStatus,
    KnowledgeBaseVersionType,
    KnowledgeBaseVersionsListResponse,
    KnowledgeBasesResponse,
    KnowledgeDeleteRequest,
    KnowledgeDocumentContentType,
    KnowledgeDocumentField,
    KnowledgeDocumentFilter,
    KnowledgeDocumentRequest,
    KnowledgeDocumentResponse,
    KnowledgeDocumentSearchRequest,
    KnowledgeDocumentSearchResponse,
    KnowledgeDocumentsResponse,
)
from .organizations import BaseOrganization, CreateOrganizationRequest, Organization, OrganizationPatchRequest
from .segments import (
    SegmentBase,
    SegmentField,
    SegmentPatchRequest,
    SegmentRequest,
    SegmentResponse,
    SegmentStatus,
    SegmentsSearchRequest,
    SegmentsSearchResponse,
)
from .translations import TranslationRequest, TranslationResponse
from .triggers import (
    EventTriggerBase,
    EventTriggerRequest,
    EventTriggerResponse,
    EventTriggerType,
    EventTriggersSearchRequest,
    EventTriggersSearchResponse,
    TriggerField,
    TriggerPartialUpdate,
)
from .users import (
    AgentUser,
    AgentUserField,
    AgentUserFilter,
    AgentUserSearchRequest,
    AgentUserSearchResponse,
    AppUserSummary,
)
from .version import __version__

__all__ = [
    "ActionBase",
    "ActionCapability",
    "ActionEnumOption",
    "ActionExecutionParamValue",
    "ActionExecutionParamValue_Attachment",
    "ActionExecutionParamValue_Primitive",
    "ActionField",
    "ActionFilter",
    "ActionFormAttachment",
    "ActionFormField",
    "ActionFormRequestParamValue",
    "ActionOAuthConfiguration",
    "ActionParameter",
    "ActionParameterType",
    "ActionRequest",
    "ActionResponse",
    "ActionsResponse",
    "ActionsSearchRequest",
    "AddDocumentFixRequest",
    "Agent",
    "AgentCapability",
    "AgentCapabilityBase",
    "AgentCapabilityField",
    "AgentCapabilityListRequest",
    "AgentCapabilityType",
    "AgentCapability_Action",
    "AgentCapability_Trigger",
    "AgentEnvironment",
    "AgentField",
    "AgentFilter",
    "AgentPatchRequest",
    "AgentUser",
    "AgentUserAnalyticsRequest",
    "AgentUserColumnDefinition",
    "AgentUserCount",
    "AgentUserField",
    "AgentUserFilter",
    "AgentUserMetric",
    "AgentUserMetric_Count",
    "AgentUserRow",
    "AgentUserSearchRequest",
    "AgentUserSearchResponse",
    "AgentUserTableRequest",
    "AgentUserTableResponse",
    "AgentsSearchRequest",
    "AgentsSearchResponse",
    "AppPrecondition",
    "AppSettings",
    "AppUser",
    "AppUserIdentifier",
    "AppUserIdentifyingPropertyType",
    "AppUserRequest",
    "AppUserResponse",
    "AppUserSummary",
    "ApplyFixesRequest",
    "AskObjectRequest",
    "AskRequest",
    "AskStreamActionEvent",
    "AskStreamChartEvent",
    "AskStreamEndEvent",
    "AskStreamMetadataEvent",
    "AskStreamOAuthButtonEvent",
    "AskStreamStartEvent",
    "AskStreamTextEvent",
    "AsyncMavenAGI",
    "AttachmentRequest",
    "AttachmentResponse",
    "AttachmentStatus",
    "BadRequestError",
    "BarChartResponse",
    "BaseAttachment",
    "BaseConversationResponse",
    "BaseKnowledgeDocument",
    "BaseOrganization",
    "BasePaginatedRequest",
    "BaseSearchRequest",
    "BotActionFormResponse",
    "BotChartResponse",
    "BotConversationMessageType",
    "BotLogic",
    "BotLogicActionExecutedDetail",
    "BotLogicActionItem",
    "BotLogicActionReviewedDetail",
    "BotLogicFormSubmissionItem",
    "BotLogicItem",
    "BotLogicItem_Actions",
    "BotLogicItem_Form",
    "BotLogicItem_Knowledge",
    "BotLogicItem_Safety",
    "BotLogicItem_Segments",
    "BotLogicItem_User",
    "BotLogicKnowledgeDetail",
    "BotLogicKnowledgeItem",
    "BotLogicSafetyItem",
    "BotLogicSegmentsItem",
    "BotLogicUserItem",
    "BotMessage",
    "BotOAuthButtonResponse",
    "BotObjectResponse",
    "BotResponse",
    "BotResponseMetadata",
    "BotResponse_ActionForm",
    "BotResponse_Chart",
    "BotResponse_OauthButton",
    "BotResponse_Object",
    "BotResponse_Text",
    "BotTextResponse",
    "BrowserInfo",
    "BrowserType",
    "Capability",
    "CategorizationResponse",
    "CellData",
    "CellData_Double",
    "CellData_Long",
    "CellData_Millisecond",
    "CellData_String",
    "ChartResponse",
    "ChartResponse_BarChart",
    "ChartResponse_DateHistogram",
    "ChartResponse_PieChart",
    "ChartSpecSchema",
    "ColumnDefinitionBase",
    "CommitAssetUploadRequest",
    "ContextInfo",
    "ConversationAnalysis",
    "ConversationAnalyticsRequest",
    "ConversationAverage",
    "ConversationBarChartRequest",
    "ConversationBasicMetric",
    "ConversationChartRequest",
    "ConversationChartRequest_BarChart",
    "ConversationChartRequest_DateHistogram",
    "ConversationChartRequest_PieChart",
    "ConversationColumnDefinition",
    "ConversationCount",
    "ConversationDateHistogramRequest",
    "ConversationDistinctCount",
    "ConversationExecutedActionPrecondition",
    "ConversationField",
    "ConversationFilter",
    "ConversationGroupBy",
    "ConversationInformation",
    "ConversationMax",
    "ConversationMedian",
    "ConversationMessageBase",
    "ConversationMessageRequest",
    "ConversationMessageResponse",
    "ConversationMessageResponse_Bot",
    "ConversationMessageResponse_User",
    "ConversationMetadata",
    "ConversationMetric",
    "ConversationMetric_Average",
    "ConversationMetric_Count",
    "ConversationMetric_DistinctCount",
    "ConversationMetric_Max",
    "ConversationMetric_Median",
    "ConversationMetric_Min",
    "ConversationMetric_Percentile",
    "ConversationMetric_Sum",
    "ConversationMin",
    "ConversationNumericMetric",
    "ConversationPatchRequest",
    "ConversationPercentile",
    "ConversationPieChartRequest",
    "ConversationPrecondition",
    "ConversationPrecondition_ActionExecuted",
    "ConversationPrecondition_App",
    "ConversationPrecondition_Metadata",
    "ConversationPrecondition_ResponseConfig",
    "ConversationPrecondition_Tags",
    "ConversationPreview",
    "ConversationRequest",
    "ConversationResponse",
    "ConversationRow",
    "ConversationSum",
    "ConversationSummary",
    "ConversationTableRequest",
    "ConversationTableResponse",
    "ConversationsResponse",
    "ConversationsSearchRequest",
    "CreateAgentRequest",
    "CreateOrganizationRequest",
    "DateHistogramResponse",
    "DeliverConversationMessageRequest",
    "DeliverMessageRequest",
    "DeliverMessageRequest_Conversation",
    "DeliverMessageRequest_User",
    "DeliverMessageResponse",
    "DeliverUserMessageRequest",
    "DeliveryStatus",
    "DeviceInfo",
    "DeviceType",
    "DocumentInformation",
    "EntityId",
    "EntityIdBase",
    "EntityIdFilter",
    "EntityIdWithoutAgent",
    "EntityResult",
    "EntityType",
    "ErrorMessage",
    "EventBaseNoId",
    "EventField",
    "EventFilter",
    "EventRequest",
    "EventRequest_SystemEvent",
    "EventRequest_UserEvent",
    "EventResponse",
    "EventResponse_SystemEvent",
    "EventResponse_UserEvent",
    "EventTriggerBase",
    "EventTriggerRequest",
    "EventTriggerResponse",
    "EventTriggerType",
    "EventTriggersSearchRequest",
    "EventTriggersSearchResponse",
    "EventType",
    "EventUserInfo",
    "EventUserInfoBase",
    "EventsSearchRequest",
    "EventsSearchResponse",
    "ExecuteCapabilityRequest",
    "ExecuteCapabilityResponse",
    "Feedback",
    "FeedbackAnalyticsRequest",
    "FeedbackBase",
    "FeedbackColumnDefinition",
    "FeedbackCount",
    "FeedbackDistinctCount",
    "FeedbackField",
    "FeedbackFilter",
    "FeedbackGroupBy",
    "FeedbackInfo",
    "FeedbackMetric",
    "FeedbackMetric_Count",
    "FeedbackMetric_DistinctCount",
    "FeedbackRequest",
    "FeedbackRow",
    "FeedbackTableRequest",
    "FeedbackTableResponse",
    "FeedbackType",
    "FieldValue",
    "FieldValue_Boolean",
    "FieldValue_DateTime",
    "FieldValue_Double",
    "FieldValue_EntityId",
    "FieldValue_Long",
    "FieldValue_Range",
    "FieldValue_String",
    "FinalizeKnowledgeBaseVersionRequest",
    "GeoInfo",
    "GroupByBase",
    "InboxFilter",
    "InboxItem",
    "InboxItemBase",
    "InboxItemDuplicateDocuments",
    "InboxItemFix",
    "InboxItemFixAddDocument",
    "InboxItemFixBase",
    "InboxItemFixDeactivateDocument",
    "InboxItemFix_AddDocument",
    "InboxItemFix_DeactivateDocument",
    "InboxItemMissingKnowledge",
    "InboxItemSeverity",
    "InboxItemStatus",
    "InboxItemType",
    "InboxItem_DuplicateDocuments",
    "InboxItem_MissingKnowledge",
    "InboxSearchRequest",
    "InboxSearchResponse",
    "InitiateAssetUploadRequest",
    "InitiateAssetUploadResponse",
    "IpInfo",
    "KnowledgeBaseField",
    "KnowledgeBaseFilter",
    "KnowledgeBaseProperties",
    "KnowledgeBaseRefreshFrequency",
    "KnowledgeBaseRequest",
    "KnowledgeBaseResponse",
    "KnowledgeBaseSearchRequest",
    "KnowledgeBaseType",
    "KnowledgeBaseVersion",
    "KnowledgeBaseVersionFinalizeStatus",
    "KnowledgeBaseVersionRequest",
    "KnowledgeBaseVersionStatus",
    "KnowledgeBaseVersionType",
    "KnowledgeBaseVersionsListResponse",
    "KnowledgeBasesResponse",
    "KnowledgeDeleteRequest",
    "KnowledgeDocumentContentType",
    "KnowledgeDocumentField",
    "KnowledgeDocumentFilter",
    "KnowledgeDocumentRequest",
    "KnowledgeDocumentResponse",
    "KnowledgeDocumentSearchRequest",
    "KnowledgeDocumentSearchResponse",
    "KnowledgeDocumentsResponse",
    "LabeledPoint",
    "LanguageInfo",
    "ListAgentCapabilitiesResponse",
    "LlmInclusionStatus",
    "LlmPersona",
    "MavenAGI",
    "MavenAGIEnvironment",
    "MessageStatus",
    "MetadataPrecondition",
    "NotFoundError",
    "NovelSystemEvent",
    "NovelSystemEventNoId",
    "NovelUserEvent",
    "NumberRange",
    "NumericConversationField",
    "ObjectStreamResponse",
    "ObjectStreamResponse_End",
    "ObjectStreamResponse_Start",
    "ObjectStreamResponse_Text",
    "Organization",
    "OrganizationPatchRequest",
    "OsInfo",
    "OsType",
    "Page",
    "PageInfo",
    "PatchAgentCapabilityRequest",
    "PieChartResponse",
    "PiiCategory",
    "Precondition",
    "PreconditionBase",
    "PreconditionGroup",
    "PreconditionGroupOperator",
    "PreconditionOperator",
    "Precondition_Conversation",
    "Precondition_Group",
    "Precondition_User",
    "Quality",
    "QualityReason",
    "Range",
    "ResolutionStatus",
    "ResponseConfig",
    "ResponseConfigPrecondition",
    "ResponseLength",
    "RowBase",
    "ScopedEntity",
    "SearchAppSettingsResponse",
    "SegmentBase",
    "SegmentField",
    "SegmentPatchRequest",
    "SegmentRequest",
    "SegmentResponse",
    "SegmentStatus",
    "SegmentsSearchRequest",
    "SegmentsSearchResponse",
    "Sentiment",
    "Series",
    "ServerError",
    "SessionInfo",
    "SimulationContext",
    "SimulationFilter",
    "Source",
    "SourceInfo",
    "SourceType",
    "StreamResponse",
    "StreamResponse_Action",
    "StreamResponse_Chart",
    "StreamResponse_End",
    "StreamResponse_Metadata",
    "StreamResponse_OauthButton",
    "StreamResponse_Start",
    "StreamResponse_Text",
    "SubmitActionFormRequest",
    "SurveyInfo",
    "SystemEvent",
    "SystemEventName",
    "TableResponseBase",
    "TagsPrecondition",
    "TimeDataPoint",
    "TimeInterval",
    "TimeSeries",
    "TranslationRequest",
    "TranslationResponse",
    "TriggerCapability",
    "TriggerField",
    "TriggerPartialUpdate",
    "TriggerType",
    "UpdateMetadataRequest",
    "UserConversationMessageType",
    "UserData",
    "UserEvent",
    "UserEventName",
    "UserMessage",
    "UserMessageBase",
    "UserMessageResponseState",
    "VisibilityType",
    "__version__",
    "actions",
    "agent_capabilities",
    "agents",
    "analytics",
    "app_settings",
    "assets",
    "commons",
    "conversation",
    "events",
    "inbox",
    "knowledge",
    "organizations",
    "segments",
    "translations",
    "triggers",
    "users",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .action_form_attachment import ActionFormAttachment

ActionFormRequestParamValue = typing.Union[typing.Optional[typing.Any], ActionFormAttachment]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id import EntityId
from ...commons.types.llm_inclusion_status import LlmInclusionStatus
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .knowledge_base_properties import KnowledgeBaseProperties
from .knowledge_base_refresh_frequency import KnowledgeBaseRefreshFrequency
from .knowledge_base_type import KnowledgeBaseType


class KnowledgeBaseResponse(KnowledgeBaseProperties):
    """
    Examples
    --------
    from mavenagi.commons import EntityId
    from mavenagi.knowledge import KnowledgeBaseResponse

    KnowledgeBaseResponse(
        knowledge_base_id=EntityId(
            reference_id="help-center",
            app_id="readme",
            organization_id="acme",
            agent_id="support",
            type="KNOWLEDGE_BASE",
        ),
        name="Help center",
        type="API",
        metadata={"key": "value"},
        tags={"tag1", "tag2"},
        refresh_frequency="DAILY",
    )
    """

    knowledge_base_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="knowledgeBaseId")] = pydantic.Field()
    """
    ID that uniquely identifies this knowledge base
    """

    active_version_id: typing_extensions.Annotated[
        typing.Optional[EntityId], FieldMetadata(alias="activeVersionId")
    ] = pydantic.Field(default=None)
    """
    ID of the knowledge base version that is currently active. Documents can be fetched using this version ID.
    """

    type: KnowledgeBaseType = pydantic.Field()
    """
    The type of the knowledge base. Can not be changed once created.
    """

    metadata: typing.Dict[str, str] = pydantic.Field()
    """
    Metadata for the knowledge base.
    """

    tags: typing.Set[str] = pydantic.Field()
    """
    The tags of the knowledge base.
    """

    llm_inclusion_status: typing_extensions.Annotated[
        typing.Optional[LlmInclusionStatus], FieldMetadata(alias="llmInclusionStatus")
    ] = pydantic.Field(default=None)
    """
    Determines whether documents in the knowledge base are sent to the LLM as part of a conversation.
    """

    refresh_frequency: typing_extensions.Annotated[
        KnowledgeBaseRefreshFrequency, FieldMetadata(alias="refreshFrequency")
    ] = pydantic.Field()
    """
    How often the knowledge base should be refreshed.
    """

    segment_id: typing_extensions.Annotated[typing.Optional[EntityId], FieldMetadata(alias="segmentId")] = (
        pydantic.Field(default=None)
    )
    """
    The IDs of the segment that must be matched for the knowledge base to be relevant to a conversation. 
    Segments are replacing inline preconditions - a Knowledge Base may not have both an inline precondition and a segment.
    Inline precondition support will be removed in a future release.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from ...commons.types.precondition_group import PreconditionGroup  # noqa: E402, F401, I001



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .action_execution_param_value import ActionExecutionParamValue
from .bot_logic_action_reviewed_detail import BotLogicActionReviewedDetail


class BotLogicActionExecutedDetail(BotLogicActionReviewedDetail):
    action_parameters: typing_extensions.Annotated[
        typing.Dict[str, ActionExecutionParamValue], FieldMetadata(alias="actionParameters")
    ]
    execution_result: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="executionResult")] = None
    execution_error: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="executionError")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ConversationMetadata(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.conversation import ConversationMetadata

    ConversationMetadata(
        metadata={
            "myapp": {"key": "newValue"},
            "conversation-owning-app": {"existingKey": "existingValue"},
        },
    )
    """

    metadata: typing.Dict[str, typing.Dict[str, str]] = pydantic.Field()
    """
    All metadata for the conversation. Keyed by appId.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
